"use strict";

const util_1 = require("./util");
const fs_extra_p_1 = require("fs-extra-p");
const httpRequest_1 = require("./httpRequest");
const _7zip_bin_1 = require("7zip-bin");
const path = require("path");
const os_1 = require("os");
const bluebird_1 = require("bluebird");
//noinspection JSUnusedLocalSymbols
const __awaiter = require("./awaiter");
let tmpDirCounter = 0;
function getTempName(prefix) {
    return `${ prefix == null ? "" : prefix + "-" }${ process.pid }-${ tmpDirCounter++ }-${ Date.now() }`;
}
const versionToPromise = new Map();
// can be called in parallel, all calls for the same version will get the same promise - will be downloaded only once
function downloadFpm(version) {
    let promise = versionToPromise.get(version);
    // if rejected, we will try to download again
    if (promise != null && !promise.isRejected()) {
        return promise;
    }
    promise = doDownloadFpm(version);
    versionToPromise.set(version, promise);
    return promise;
}
exports.downloadFpm = downloadFpm;
function doDownloadFpm(version) {
    return __awaiter(this, void 0, void 0, function* () {
        const dirName = `fpm-${ version }-osx`;
        const url = `https://github.com/develar/fpm-self-contained/releases/download/v${ version }/${ dirName }.7z`;
        const cache = path.join(__dirname, "..", "node_modules", ".fpm");
        const fpmDir = path.join(cache, dirName);
        const stat = yield util_1.statOrNull(fpmDir);
        if (stat != null && stat.isDirectory()) {
            return path.join(fpmDir, "fpm");
        }
        // the only version currently supported (i.e. all clients are consumed the same version
        yield fs_extra_p_1.emptyDir(cache);
        const archiveName = path.join(os_1.tmpdir(), getTempName("fpm-download") + ".7z");
        yield httpRequest_1.download(url, archiveName);
        const tempUnpackDir = path.join(cache, getTempName());
        yield util_1.spawn(_7zip_bin_1.path7za, ["x", archiveName, "-o" + tempUnpackDir, "-bb" + (util_1.debug.enabled ? "3" : "0"), "-bd"], {
            cwd: cache,
            stdio: ["ignore", "inherit", "inherit"]
        });
        yield fs_extra_p_1.move(path.join(tempUnpackDir, dirName), fpmDir, { clobber: true });
        yield bluebird_1.Promise.all([fs_extra_p_1.remove(tempUnpackDir), fs_extra_p_1.remove(archiveName)]);
        return path.join(fpmDir, "fpm");
    });
}
//# sourceMappingURL=fpmDownload.js.map